"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getPersistedConfig = exports.setPersistedConfig = exports.get$select = exports.restore$select = exports.hide$select = exports.checkMulti = exports.throwUnlessCan = exports.getAbility = exports.getAppOptions = exports.makeDefaultOptions = exports.makeOptions = void 0;
var get_1 = __importDefault(require("lodash/get"));
var set_1 = __importDefault(require("lodash/set"));
var unset_1 = __importDefault(require("lodash/unset"));
var errors_1 = require("@feathersjs/errors");
var feathers_utils_1 = require("feathers-utils");
var makeOptions = function (app, options) {
    var _a;
    options = options || {};
    var appOptions = ((_a = app === null || app === void 0 ? void 0 : app.get("casl")) === null || _a === void 0 ? void 0 : _a.authorizeHook) || exports.makeDefaultOptions();
    return Object.assign(appOptions, options);
};
exports.makeOptions = makeOptions;
var makeDefaultOptions = function (options) {
    options = options || {};
    options.checkMultiActions = false;
    return options;
};
exports.makeDefaultOptions = makeDefaultOptions;
var getAppOptions = function (app) {
    var caslOptions = app.get("casl");
    if (caslOptions === null || caslOptions === void 0 ? void 0 : caslOptions.authorizeHook) {
        return caslOptions.authorizeHook;
    }
};
exports.getAppOptions = getAppOptions;
var getAbility = function (context, options) {
    var _a;
    options = options || {};
    if (options === null || options === void 0 ? void 0 : options.ability) {
        if (typeof options.ability === "function") {
            var ability = options.ability(context);
            return Promise.resolve(ability);
        }
        else {
            return Promise.resolve(options.ability);
        }
    }
    if ((_a = context === null || context === void 0 ? void 0 : context.params) === null || _a === void 0 ? void 0 : _a.ability) {
        return Promise.resolve(context.params.ability);
    }
    return Promise.resolve(undefined);
};
exports.getAbility = getAbility;
var move = function (context, fromPath, toPath) {
    var val = get_1.default(context, fromPath);
    if (val !== undefined) {
        unset_1.default(context, fromPath);
        set_1.default(context, toPath, val);
    }
    return val;
};
var throwUnlessCan = function (ability, method, resource, modelName, actionOnForbidden) {
    if (ability.cannot(method, resource)) {
        if (actionOnForbidden)
            actionOnForbidden();
        throw new errors_1.Forbidden("You are not allowed to " + method + " " + modelName);
    }
};
exports.throwUnlessCan = throwUnlessCan;
var checkMulti = function (context, ability, modelName, actionOnForbidden) {
    var method = context.method;
    var currentIsMulti = feathers_utils_1.isMulti(context);
    if (!currentIsMulti) {
        return true;
    }
    if ((method === "find" && ability.can(method, modelName)) ||
        (ability.can(method + "-multi", modelName))) {
        return true;
    }
    if (actionOnForbidden)
        actionOnForbidden();
    throw new errors_1.Forbidden("You're not allowed to multi-" + method + " " + modelName);
};
exports.checkMulti = checkMulti;
var hide$select = function (context) {
    return move(context, "params.query.$select", "params.casl.$select");
};
exports.hide$select = hide$select;
var restore$select = function (context) {
    move(context, "params.casl.$select", "params.query.$select");
    return get_1.default(context, "params.query.$select");
};
exports.restore$select = restore$select;
var get$select = function (context) {
    return exports.getPersistedConfig(context, "$select");
};
exports.get$select = get$select;
var setPersistedConfig = function (context, key, val) {
    return set_1.default(context, "params.casl." + key, val);
};
exports.setPersistedConfig = setPersistedConfig;
var getPersistedConfig = function (context, key) {
    return get_1.default(context, "params.casl." + key);
};
exports.getPersistedConfig = getPersistedConfig;
