"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var isPlainObject_1 = __importDefault(require("lodash/isPlainObject"));
var errors_1 = require("@feathersjs/errors");
var feathers_utils_1 = require("feathers-utils");
var invertedMap = {
    "$gt": "$lte",
    "$gte": "$lt",
    "$lt": "$gte",
    "$lte": "$gt",
    "$in": "$nin",
    "$nin": "$in",
    "$ne": function (prop) {
        return prop["$ne"];
    }
};
var invertedProp = function (prop, name) {
    var _a;
    var map = invertedMap[name];
    if (typeof map === "string") {
        return _a = {}, _a[map] = prop[name], _a;
    }
    else if (typeof map === "function") {
        return map(prop);
    }
};
var convertRuleToQuery = function (rule, options) {
    var conditions = rule.conditions, inverted = rule.inverted;
    if (!conditions) {
        if (inverted && (options === null || options === void 0 ? void 0 : options.actionOnForbidden)) {
            options.actionOnForbidden();
        }
        return {};
    }
    if (inverted) {
        var newConditions = {};
        for (var prop in conditions) {
            if (isPlainObject_1.default(conditions[prop])) {
                var obj = conditions[prop];
                for (var name_1 in obj) {
                    if (![
                        "$gt",
                        "$gte",
                        "$lt",
                        "$lte",
                        "$in",
                        "$nin",
                        "$ne"
                    ].includes(name_1)) {
                        console.error("CASL: not supported property: " + name_1);
                        continue;
                    }
                    newConditions[prop] = invertedProp(obj, name_1);
                }
            }
            else {
                newConditions[prop] = { $ne: conditions[prop] };
            }
        }
        return newConditions;
    }
    else {
        return conditions;
    }
};
exports.default = (function (ability, method, subject, options) {
    options = options || {};
    options.actionOnForbidden = options.actionOnForbidden || (function () {
        throw new errors_1.Forbidden("You're not allowed to make this request");
    });
    var rules = ability.rulesFor(method, subject);
    var query = {};
    for (var i = 0; i < rules.length; i++) {
        var rule = rules[i];
        var currentQuery = convertRuleToQuery(rule, options);
        query = feathers_utils_1.mergeQuery(query, currentQuery, { defaultHandle: "combine" });
    }
    return query;
});
